@@ -1,36 +1,53 @@
+# create persistent Peekers?
 package ObjStore::Peeker;
 use strict;
 use Carp;
-use vars qw($LinePrefix $LineIndent $LineSep
-	    $Addr $SummaryWidth $MaxWidth $MaxDepth $MaxInstances
-	    $To $All);
-
-$LinePrefix='';
-$LineIndent='  ';
-$LineSep="\n";
-$Addr=0;
-$SummaryWidth=3;
-$MaxWidth=20;
-$MaxDepth=20;
-$MaxInstances=3;
-$To='string';
-$All=0;
+use vars qw{
+    $vareq
+    $prefix
+	$indent
+	    $sep
+		$addr
+		    $refcnt
+			$summary_width
+			    $instances
+				$width
+				    $depth
+					$to
+					    $all
+					    };
+$vareq=0;
+$prefix='';
+$indent='  ';
+$sep="\n";
+$addr=0;
+$refcnt=0;
+$summary_width=3;
+$instances=3;
+$width=20;
+$depth=20;
+$to='string';
+$all=0;
 
 sub new {
-    my ($class, %cnf) = @_;
+    my ($class, @opts) = @_;
     my $o = bless {
-	prefix => $LinePrefix,
-	indent => $LineIndent,
-	sep => $LineSep,
-	addr => $Addr,
-	summary_width => $SummaryWidth,
-	instances => $MaxInstances,
-	width => 20,
-	depth => 20,
-	to => $To,
-	all => $All,
+	serial => 1,
+	vareq => 0,
+	prefix => $prefix,
+	indent => $indent,
+	sep => $sep,
+	addr => $addr,
+	refcnt => $refcnt,
+	summary_width => $summary_width,
+	instances => $instances,
+	width => $width,
+	depth => $depth,
+	to => $to,
+	all => $all,
     }, $class;
-    while (my ($k,$v) = each %cnf) { $o->{$k} = $v; }
+    croak "Odd number of parameters" if @opts & 1;
+    while (@opts) { $o->{pop @opts} = pop @opts }
     $o;
 }
 
@@ -42,11 +59,14 @@ sub Peek {
     $o->{has_sep} = 0;
     $o->{has_prefix} = 0;
     $o->{coverage} = 0;
+    $o->o('$fake'.$o->{serial}." = ") if $o->{vareq};
     $o->_peek($top);
+    $o->nl;
+    ++ $o->{serial};
     $o->{output};
 }
 
-sub PercentUnused { die "wildy inaccurate metric unsupported"; }
+sub PercentUnused { die "wildy inaccurate metric no longer supported"; }
 
 sub Coverage {
     my ($o) = @_;
@@ -72,8 +92,14 @@ sub o {
     $o->{has_sep}=0;
     $o->{has_prefix}=0;
     my $t = ref $o->{to};
-    if (!$t and $o->{to} eq 'string') {
-	$o->{output} .= join('', @_);
+    if (!$t) {
+	if ($o->{to} eq 'string') {
+	    $o->{output} .= join('', @_);
+	} elsif ($o->{to} eq 'stdout') {
+	    for (@_) { print };
+	} else {
+	    die "ObjStore::Peeker: Don't know how to write to $o->{to}";
+	}
     } elsif ($t eq 'CODE') {
 	$o->{to}->(@_);
     } elsif ($t->isa('IO::Handle') or $t->isa('FileHandle')) {
@@ -93,16 +119,19 @@ sub avg {
 
 sub _peek {
     my ($o, $val) = @_;
+
+    # interrogate
     my $type = ObjStore::reftype $val;
     my $class = ref $val;
+    my $blessed = $class ne $type;
 
-    # Since persistent blessing is likely turned off...
-    $class = $val->_ref if ($class and $class->isa('ObjStore::UNIVERSAL'));
+    # Since persistent-bless might be tweaked...
+    $class = $val->_blessed_to if ($class and $class->isa('ObjStore::UNIVERSAL'));
 
     if (!$class) {
 	if (!defined $val) {
 	    $o->o('undef,');
-	} elsif ($val =~ /^-?[1-9]\d{0,8}$/) {
+	} elsif ($val =~ /^-?[0-9]\d{0,8}$/) {   #floating point? XXX
 	    $o->o("$val,");
 	} else {
 	    $val =~ s/([\\\'])/\\$1/g;
@@ -114,19 +143,15 @@ sub _peek {
 
     my $addr = "$val";
     my $name = $o->{addr} ? $addr : $class;
+    if ($o->{refcnt} and $blessed and $val->can("_refcnt")) {
+	$name .= " (".join(',', $val->_refcnt).")";
+    }
     $o->{seen}{$class} ||= 0;
 
     if ($o->{level} > $o->{depth} or defined($o->{seen}{$addr}) or
-	($class !~ /^ObjStore::/ and $o->{seen}{$class} > $o->{instances})) {
-	if ($type eq 'HASH') {
-	    $o->o("$name { ... }");
-	} elsif ($type eq 'ARRAY') {
-	    $o->o("$name [ ... ]");
-	} elsif ($class->isa('ObjStore::Set')) {
-	    $o->o("$name [ ... ]");
-	} else {
-	    $o->o("$class ...");
-	}
+	($blessed and $val->can('_is_blessed') and
+	 $val->_is_blessed and $o->{seen}{$class} > $o->{instances})) {
+	$o->o("$name ...");
 	++ $o->{coverage};
 	return;
     }
@@ -134,50 +159,10 @@ sub _peek {
     ++ $o->{seen}{$class};
 
     $o->prefix;
-    if ($class eq 'ObjStore::Database') {
-	for my $r ($val->get_all_roots) {
-	    next if (!$o->{all} and $r->get_name eq ObjStore::_PRIVATE_ROOT);
-
-	    $o->prefix;
-	    ++$o->{level};
-	    my $name = $o->{addr}? "$r" : 'ObjStore::Root';
-	    $o->o("$name ",$r->get_name," = ");
-	    $o->{has_prefix}=1;
-	    $o->_peek($r->get_value);
-	    --$o->{level};
-	    $o->nl;
-	    ++ $o->{coverage};
-	}
-    } elsif ($class->isa('ObjStore::Set')) {
-	my @S;
-	my $x=0;
-	for (my $v=$val->first; $v; $v=$val->next) {
-	    ++ $o->{coverage};
-	    last if $x++ > $o->{width}+1;
-	    push(@S, $v);
-	}
-	my $big = @S > $o->{width};
-	my $limit = $big ? $o->{summary_width}-1 : @S;
-
-	$o->o($name . " [");
-	$o->nl;
-	++$o->{level};
-	for (my $v=$val->first; $v; $v=$val->next) {
-	    last if $limit-- <= 0;
-	    $o->prefix;
-	    $o->_peek($v);
-	    $o->nl;
-	}
-	if ($big) {
-	    $o->prefix;
-	    $o->o("...");
-	    $o->nl;
-	}
-	--$o->{level};
-	$o->prefix;
-	$o->o("],");
-	$o->nl;
+    if ($blessed and $val->can('_peek')) {
+	$val->_peek($o, $name);
     } elsif ($type eq 'HASH') {
+	# might try to use _count method XXX
 	my @S;
 	my $x=0;
 	while (my($k,$v) = each %$val) {
@@ -211,7 +196,7 @@ sub _peek {
 	$o->o("},");
 	$o->nl;
     } elsif ($type eq 'ARRAY') {
-	my $len = $val->can("_LENGTH")? $val->_LENGTH : @$val;
+	my $len = ($blessed and $val->can("_count"))? $val->_count : @$val;
 	$o->{coverage} += $len;
 	my $big = $len > $o->{width};
 	my $limit = $big? $o->{summary_width} : $len;
@@ -233,15 +218,6 @@ sub _peek {
 	$o->prefix;
 	$o->o("],");
 	$o->nl;
-    } elsif ($class->isa('ObjStore::Cursor')) {
-	++ $o->{coverage};
-	$o->o("$name => ");
-	$o->{has_prefix}=1;
-	++ $o->{level};
-	my $at = $val->focus;
-	$o->_peek($at);
-	-- $o->{level};
-	$o->nl;
     } elsif ($type eq 'SCALAR') {
 	++ $o->{coverage};
 	$o->o($name);
